#include <LiquidCrystal.h>

//Pins for LED
/* RGB values are from 0 to 255. Certain colours might be different 
 * numbers depening on how your RGB LED strip works.
 * The strip we are using turns red when the values are: 255, 255, 255*/
int redPin = 10;
int bluePin = 11;
int greenPin = 9;

//variables for ulstrasonic sensor
int trigPin = 4;
int echoPin = 7;
long duration;

//Pins for LCD
//Parameters: (rs, enable, d4, d5, d6, d7)
LiquidCrystal lcd(6, 2, 3, 8, 12, 13);

//speaker variables
int NOTE_FS4 = 370;
int NOTE_C5 = 523;
int speakerPin = 5;

//variables for photoresistor
boolean isDark;
int photoPin = A0;             

int delaySpeed = 500;

void setup() {
  //Set up LED pin
  pinMode(redPin, OUTPUT);
  pinMode(bluePin, OUTPUT);
  pinMode(greenPin, OUTPUT);

  //Set up ultrasonic sensor
  pinMode(echoPin, INPUT);
  pinMode(trigPin, OUTPUT);
  
  //set up lcd
  //16 and 2 is number of rows and columns
  lcd.begin(16, 2);

  //set up speaker
  pinMode(speakerPin, OUTPUT);

  //set up photoresistor
  pinMode(photoPin, INPUT);
  Serial.begin(9600);
  Serial.println("hey, how u doing?");

}

void loop() {
  isDark = determineDark();
  int distance = getDistance();
  
  Serial.print("Distance: ");
  Serial.println(distance);
  
  lcd.setCursor(0,0);
  if (!isDark) {
    if (distance >= 500) {
      setColor(0, 255, 0); //LED strip turns green
      writeScreen("hi, want to be");
      lcd.setCursor(0,1);
      writeScreen("friends?");
      Serial.println("hi, want to be friends?");
      delaySpeed = 500;
    }
    else if (distance >= 50 && distance < 500) {
      setColor(100, 150, 50); //LED strip turns light blue
      writeScreen("Come closer");
      Serial.println("come closer");
      delaySpeed = 500;
    }
    else {
      setColor(255, 255, 255); //LED strip turns red
      writeScreen("too close!");
      Serial.println("too close!");
      delaySpeed = 500;
    }
  }
  else {
    if (distance >= 500) {
        setColor(100, 200, 0); //LED strip turns light green
        writeScreen("hi, come join me :)");
        Serial.println("hi, come join me");
        delaySpeed = 500;
    }
    else if (distance >= 50 && distance < 500) {
        setColor(255, 20, 10); //LED strip turns magenta
        writeScreen("I need space");
        Serial.println("i need space");
        delaySpeed = 500;
    }
    else{
      setColor(255,255,255); //LED strip turns red
      delay(300);
      setColor(100,0,255); //LED strip turns blue
      writeScreen("eff off");
      Serial.println("eff off");
      delaySpeed = 300; //delay speed is changed so it can flash between red and blue faster
      sing("siren");
    }
  }
  
  delay(delaySpeed);
  lcd.clear();
}

// methods
boolean determineDark() {
  int darknessVal = analogRead(photoPin);
  if(darknessVal < 250){
    return true;
  }
  Serial.println(darknessVal);
  return false;
}

void sing(String song){
  if(song == "siren"){
    Serial.println("Siren:");
    tone(speakerPin, NOTE_FS4, 500);
    delay(100);
    tone(speakerPin, NOTE_C5, 500);
    delay(100);
  }
}

void setColor(int red, int green, int blue) {
  analogWrite(redPin, red);
  analogWrite(greenPin, green);
  analogWrite(bluePin, blue);
}

void writeScreen(String message) {
  lcd.print(message);
}

//reference code from: https://howtomechatronics.com/tutorials/arduino/ultrasonic-sensor-hc-sr04/
int getDistance(){
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  // Sets the trigPin on HIGH state for 10 micro seconds
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
  // Reads the echoPin, returns the sound wave travel time in microseconds
  delayMicroseconds(10);
  duration = pulseIn(echoPin, HIGH);
  // Calculating the distance
  return duration*0.034/2;
}
